"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createEventHandler = void 0;
var _error = _interopRequireDefault(require("@webiny/error"));
var _dynamodb = require("aws-sdk/clients/dynamodb");
var _apiElasticsearch = require("@webiny/api-elasticsearch");
var _handlerAws = require("@webiny/handler-aws");
var _pRetry = _interopRequireDefault(require("p-retry"));
var Operations = /*#__PURE__*/function (Operations) {
  Operations["INSERT"] = "INSERT";
  Operations["MODIFY"] = "MODIFY";
  Operations["REMOVE"] = "REMOVE";
  return Operations;
}(Operations || {});
const getError = item => {
  if (!item.index || !item.index.error || !item.index.error.reason) {
    return null;
  }
  const reason = item.index.error.reason;
  if (reason.match(/no such index \[([a-zA-Z0-9_-]+)\]/) !== null) {
    return "index";
  }
  return reason;
};
const getNumberEnvVariable = (name, def) => {
  const input = process.env[name];
  const value = Number(input);
  if (value > 0) {
    return value;
  }
  return def;
};
const checkErrors = result => {
  if (!result || !result.body || !result.body.items) {
    return;
  }
  for (const item of result.body.items) {
    const err = getError(item);
    if (!err) {
      continue;
    } else if (err === "index") {
      if (process.env.DEBUG === "true") {
        console.error("Bulk response", JSON.stringify(result, null, 2));
      }
      continue;
    }
    console.error(item.error);
    throw new _error.default(err, "DYNAMODB_TO_ELASTICSEARCH_ERROR", item);
  }
};
const createEventHandler = () => {
  return (0, _handlerAws.createDynamoDBEventHandler)(async ({
    event,
    context: ctx
  }) => {
    const context = ctx;
    if (!context.elasticsearch) {
      console.error("Missing elasticsearch definition on context.");
      return null;
    }

    /**
     * Wrap the code we need to run into the function, so it can be called within itself.
     */
    const execute = async () => {
      const operations = [];
      for (const record of event.Records) {
        const dynamodb = record.dynamodb;
        if (!dynamodb) {
          continue;
        }
        const newImage = _dynamodb.Converter.unmarshall(dynamodb.NewImage);
        if (newImage.ignore === true) {
          continue;
        }
        const oldImage = _dynamodb.Converter.unmarshall(dynamodb.OldImage);
        const keys = _dynamodb.Converter.unmarshall(dynamodb.Keys);
        const _id = `${keys.PK}:${keys.SK}`;
        const operation = record.eventName;

        /**
         * On operations other than REMOVE we decompress the data and store it into the Elasticsearch.
         * No need to try to decompress if operation is REMOVE since there is no data sent into that operation.
         */
        let data = undefined;
        if (operation !== Operations.REMOVE) {
          /**
           * We must decompress the data that is going into the Elasticsearch.
           */
          data = await (0, _apiElasticsearch.decompress)(context.plugins, newImage.data);
          /**
           * No point in writing null or undefined data into the Elasticsearch.
           * This might happen on some error while decompressing. We will log it.
           *
           * Data should NEVER be null or undefined in the Elasticsearch DynamoDB table, unless it is a delete operations.
           * If it is - it is a bug.
           */
          if (data === undefined || data === null) {
            console.error(`Could not get decompressed data, skipping ES operation "${operation}", ID ${_id}`);
            continue;
          }
        }
        switch (record.eventName) {
          case Operations.INSERT:
          case Operations.MODIFY:
            operations.push({
              index: {
                _id,
                _index: newImage.index
              }
            }, data);
            break;
          case Operations.REMOVE:
            operations.push({
              delete: {
                _id,
                _index: oldImage.index
              }
            });
            break;
          default:
            break;
        }
      }
      if (!operations.length) {
        return;
      }
      try {
        const res = await context.elasticsearch.bulk({
          body: operations
        });
        checkErrors(res);
      } catch (error) {
        if (process.env.DEBUG === "true") {
          const meta = (error === null || error === void 0 ? void 0 : error.meta) || {};
          delete meta["meta"];
          console.error("Bulk error", JSON.stringify(error, null, 2));
        }
        throw error;
      }
    };
    const maxRetryTime = getNumberEnvVariable("WEBINY_DYNAMODB_TO_ELASTICSEARCH_MAX_RETRY_TIME", 300000);
    const retries = getNumberEnvVariable("WEBINY_DYNAMODB_TO_ELASTICSEARCH_RETRIES", 20);
    const minTimeout = getNumberEnvVariable("WEBINY_DYNAMODB_TO_ELASTICSEARCH_MIN_TIMEOUT", 1500);
    const maxTimeout = getNumberEnvVariable("WEBINY_DYNAMODB_TO_ELASTICSEARCH_MAX_TIMEOUT", 30000);
    await (0, _pRetry.default)(execute, {
      maxRetryTime,
      retries,
      minTimeout,
      maxTimeout,
      onFailedAttempt: error => {
        /**
         * We will only log attempts which are after 3/4 of total attempts.
         */
        if (error.attemptNumber < retries * 0.75) {
          return;
        }
        console.error(`Attempt #${error.attemptNumber} failed.`);
        console.error(error.message);
      }
    });
    return null;
  });
};
exports.createEventHandler = createEventHandler;

//# sourceMappingURL=index.js.map